"use strict";
this.name = "SE-main-script";
this.author = "Ramirez & Svengali";
this.copyright = "March 2011";
this.description = "Special events for market prices";

//2.8 updated to work with MarketScriptInterface.oxp
//2.6.1 added system marker -spara-
//2.6 fixed to work with Oolite 1.82 -spara-

this.startUp = function () {
	var harakiri = false;
	// Check for GNN and react if not found...
	if (!worldScripts.GNN) harakiri = true;
	else if (!this.compareVersion(worldScripts.GNN.version, "1.0")) harakiri = true;
	if (harakiri) {
		player.consoleMessage("BlOomberg - Minimum GNN version (1.0) not detected. Disabling...", 10);
		log(this.name, this.name + ': Minimum GNN version (1.0) not detected. Disabling all functionality in BlOomberg.');
		for (prop in this) { if (prop !== 'name' && prop !== 'version') delete this[prop]; }
		return;
	}
	delete this.compareVersion;
	this.logging = false; // Must be false for release
	this.checkShort = false; // Use short time check. Must be false for release
	if (this.logging) this.getLog('Loaded:');

	var msi = worldScripts.MarketScriptInterface_Main;
	msi.$addMarketInterface("system_local", "$updateLocalCommodityDefinition", this.name);
}

this.clearVars = function () {
	missionVariables.SE_commodity = null;
	missionVariables.SE_deadline = null;
	missionVariables.SE_deviation = null;
	missionVariables.SE_economy_type = null;
	missionVariables.SE_galaxy_number = null;
	missionVariables.SE_missiontext_string = null;
	missionVariables.SE_set = null;
	missionVariables.SE_status = null;
	missionVariables.SE_system_economy = null;
	missionVariables.SE_system_ID = null;
	missionVariables.SE_system_name = null;

	//used in bloomberg_market.js -spara-
	missionVariables.SE_market_key = null;

	return;
}

this.compareVersion = function (strA, strB) {
	var x = strA.split('.'), y = strB.split('.');
	var xl = x.length, yl = y.length;
	for (let i = 0; i < yl; i++) {
		if (xl <= i) return (false);
		if (parseInt(x[i]) > y[i]) break;
		if (parseInt(x[i]) < y[i]) return (false);
	}
	return (true);
}

// Check if screens should be passed or event should be reset
this.guiScreenChanged = function () {
	if (player.ship.docked && guiScreen === "GUI_SCREEN_MARKET") this.prepareScreen();
}

this.shipExitedWitchspace = function () {
	if (missionVariables.SE_status === "RUNNING" && clock.days > missionVariables.SE_deadline) this.prepareScreen();
}

this.shipWillEnterWitchspace = function (type) {
	if (type === "galactic jump") this.resetSpecialEvent();
}

// Prepare screen for Snoopers
this.prepareScreen = function () {
	if (!missionVariables.SE_status) {
		var event_delay = Math.floor(Math.random() * 10) + 20;
		if (this.checkShort) missionVariables.SE_set = clock.days;
		else missionVariables.SE_set = clock.days + event_delay;
		missionVariables.SE_status = "SET";
	}
	if (this.logging) log(this.name, this.name + ': Set is:' + missionVariables.SE_set);
	var obj = new Object();
	if (missionVariables.SE_status === "RUNNING") {
		if (this.logging) log(this.name, this.name + ': Deadline:' + missionVariables.SE_deadline);
		if (clock.days >= missionVariables.SE_deadline) {
			obj = { ID: this.name, Message: expandMissionText("SE_reset_event"), Pic: "SE-bloomberg.png", Music: "SE-stockmarket.ogg", Priority: 1 };
			this.passScreen(obj, null);
		}
		return;
	} else if (clock.days >= missionVariables.SE_set && missionVariables.SE_status === "SET") {
		missionVariables.SE_system_ID = (Math.floor(Math.random() * 255));
		missionVariables.SE_galaxy_number = galaxyNumber;
		missionVariables.SE_system_name = System.systemNameForID([missionVariables.SE_system_ID]);
		missionVariables.SE_system_economy = System.infoForSystem([missionVariables.SE_galaxy_number], [missionVariables.SE_system_ID]).economy;
		if (missionVariables.SE_system_economy > 3) {
			missionVariables.SE_economy_type = "agricultural";
			missionVariables.SE_commodity = expandDescription("[SE_agricultural_commodity]");
			missionVariables.SE_deviation = expandDescription("[SE_" + [missionVariables.SE_commodity] + "_" + [missionVariables.SE_economy_type] + "_" + "deviation" + "]");
			missionVariables.SE_missiontext_string = "SE_" + [missionVariables.SE_commodity] + "_" + [missionVariables.SE_deviation];
		} else {
			missionVariables.SE_economy_type = "industrial";
			missionVariables.SE_commodity = expandDescription("[SE_industrial_commodity]");
			missionVariables.SE_deviation = expandDescription("[SE_" + [missionVariables.SE_commodity] + "_" + [missionVariables.SE_economy_type] + "_" + "deviation" + "]");
			missionVariables.SE_missiontext_string = "SE_" + [missionVariables.SE_commodity] + "_" + [missionVariables.SE_deviation];
		}
		obj = { ID: this.name, Message: expandMissionText([missionVariables.SE_missiontext_string]), Pic: "SE-bloomberg.png", Music: "SE-stockmarket.ogg", Priority: 2 };
		if (this.logging) log(this.name, JSON.stringify(obj));
		this.passScreen(obj, 1);
	} else if (this.logging) log(this.name, 'Delay.');
	return;
}

// Pass the prepared object to GNN and set (or reset) variables
this.passScreen = function (obj, mode) {
	if (this.logging) this.getLog('Status prepared:');
	// Pass the object and get errorcode back
	var a = worldScripts.GNN._insertNews(obj);
	if (!a) { // On success
		if (mode) missionVariables.SE_status = "INJECTED"; // Set flag for inserted message
		if (this.logging) log(this.name, this.name + ': Event passed. Status:' + missionVariables.SE_status);
	} else {
		if (a < 0) { // If CRC is still stored or buffers full, add 2 more days and try again
			if (missionVariables.SE_status === "SET") missionVariables.SE_set = parseInt(missionVariables.SE_set) + 2;
			if (missionVariables.SE_status === "RUNNING") missionVariables.SE_deadline = parseInt(missionVariables.SE_deadline) + 2;
			return;
		}
		if (this.logging) log(this.name, 'GNN returned:' + a);
		this.resetSpecialEvent(); // On failure reset everything...
	}
	if (this.logging) this.getLog('Status after insertion:');
	return;
}

// Callback from GNN
this.newsDisplayed = function (what) { worldScripts["SE-main-script"].newsReaction(what); return; }

this.newsReaction = function (what) {
	if (missionVariables.SE_galaxy_number !== null) {
		if (missionVariables.SE_status === "INJECTED") {
			missionVariables.SE_market_key = "SE-station-" + [missionVariables.SE_commodity] + "-" + [missionVariables.SE_deviation];

			var mi = expandMissionText("SE_market_information");
			if (this.logging) log(this.name, "Mission: " + mi);
			mission.setInstructions(mi, this.name);
			mission.markSystem({
				system: missionVariables.SE_system_ID,
				name: this.name,
				markerColor: "blueColor",
				markerShape: "MARKER_DIAMOND"
			});
			var event_duration = Math.floor(Math.random() * 8) + 12;
			if (this.checkShort) missionVariables.SE_deadline = clock.days + 1;
			else missionVariables.SE_deadline = clock.days + event_duration;
			missionVariables.SE_status = "RUNNING";
			if (this.logging) log(this.name, this.name + ': Event set. Duration until:' + missionVariables.SE_deadline);
		} else if (missionVariables.SE_status === "RUNNING") this.resetSpecialEvent();
	}
	if (this.logging) this.getLog('Status after display:');
	return;
}

this.resetSpecialEvent = function () {
	// Can be undefined if status=SET but not yet displayed
	if (missionVariables.SE_set && missionVariables.SE_galaxy_number !== null) {
		mission.unmarkSystem({ system: missionVariables.SE_system_ID, name: this.name });
	}
	mission.setInstructions(null, this.name);
	this.clearVars();
	if (this.logging) this.getLog('Status after cleaning:');
	return;
}

this.getLog = function (m) {
	log(this.name, m);
	for (let prop in missionVariables) { if (prop.substring(0, 2) === 'SE_') log(this.name, "mV[" + prop + "] : " + missionVariables[prop]); }
	return;
}

this.$originalDefs = {
	"SE-station-Alloys-decrease" : {
		"alloys" : [0,0,42,1,1,17,31,31,0]
	},
	"SE-station-Alloys-increase" : {
		"alloys" : [0,0,150,1,1,17,31,31,0]
	},
	"SE-station-Computers-decrease" : {
		"computers" : [0,0,60,14,14,56,3,3,0]
	},
	"SE-station-Computers-increase" : {
		"computers" : [0,0,200,14,14,56,3,3,0]
	},
	"SE-station-Firearms-decrease" : {
		"firearms" : [0,0,10,13,1,60,7,3,0]
	},
	"SE-station-Firearms-increase" : {
		"firearms" : [0,0,155,13,10,50,7,7,0]
	},
	"SE-station-Food-decrease" : {
		"food" : [0,0,12,-1,2,25,1,1,0]
	},
	"SE-station-Furs-decrease" : {
		"furs" : [0,0,100,-9,-9,220,63,63,0]
	},
	"SE-station-Furs-increase" : {
		"furs" : [0,0,220,-9,-9,240,10,63,0]
	},
	"SE-station-Gem-stones-decrease" : {
		"gem_stones" : [0,0,25,-1,-1,250,15,15,2]
	},
	"SE-station-Gem-stones-increase" : {
		"gem_stones" : [0,0,90,-1,-1,250,15,15,2]
	},
	"SE-station-Gold-decrease" : {
		"gold" : [0,0,40,-1,-1,66,7,7,1]
	},
	"SE-station-Gold-increase" : {
		"gold" : [0,0,190,-1,-1,66,7,7,1]
	},
	"SE-station-Liquor/Wines-decrease" : {
		"liquor_wines" : [0,0,64,-5,-5,251,15,15,0]
	},
	"SE-station-Liquor/Wines-increase" : {
		"liquor_wines" : [0,0,180,-5,-5,251,15,15,0]
	},
	"SE-station-Minerals-decrease" : {
		"minerals" : [0,0,10,-1,-1,53,3,3,0]
	},
	"SE-station-Minerals-increase" : {
		"minerals" : [0,0,135,-1,-1,53,3,3,0]
	},
	"SE-station-Narcotics-decrease" : {
		"narcotics" : [0,0,10,5,5,40,20,5,0]
	},
	"SE-station-Platinum-decrease" : {
		"platinum" : [0,0,80,-2,-2,55,31,31,1]
	},
	"SE-station-Platinum-increase" : {
		"platinum" : [0,0,220,-2,-2,55,31,31,1]
	},
	"SE-station-Radioactives-decrease" : {
		"radioactives" : [0,0,20,-3,-3,80,7,7,0]
	},
	"SE-station-Radioactives-increase" : {
		"radioactives" : [0,0,150,-3,-3,2,7,7,0]
	},
	"SE-station-Slaves-decrease" : {
		"slaves" : [0,0,40,-5,-5,20,31,31,0]
	},
	"SE-station-Slaves-increase" : {
		"slaves" : [0,0,230,-5,-5,226,31,31,0]
	},
	"SE-station-Textiles-decrease" : {
		"textiles" : [0,0,5,-1,-1,10,3,3,0]
	},
	"SE-station-Textiles-increase" : {
		"textiles" : [0,0,100,-1,-1,10,3,3,0]
	}
}

this.$updateLocalCommodityDefinition = function(goodDefinition, station, systemID) {
	// even though defined at the system level, this will only apply to one system - wherever the event is taking place
	if (systemID && galaxyNumber == [missionVariables.SE_galaxy_number] && systemID == [missionVariables.SE_system_ID]) {
		var commodity = goodDefinition.key;
		//mission variable is set in the main script
		var oldDefs = this.$originalDefs[missionVariables.SE_market_key][commodity];

		//old style definition found for the good. calculate it the old way
		//only the price is changed
		if (oldDefs) {
			var market_base_price = oldDefs[2];
			var market_eco_adjust_price = oldDefs[3];
			var market_mask_price = oldDefs[6];
			var market_rnd = Math.floor(Math.random() * 256);

			var economy = system.economy;
			
			var price = (market_base_price + (market_rnd & market_mask_price) + (economy * market_eco_adjust_price)) & 255;
			price *= 0.4;

			goodDefinition.price = price * 10;
		}
	}
	return goodDefinition;
};
